@@ -1,55 +1,28 @@
 /*
+ * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @license AGPL-3.0
  *
- *    OPEN-XCHANGE legal information
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *    All intellectual property rights in the Software are protected by
- *    international copyright laws.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
  *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with OX App Suite.  If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
  *
- *    In some countries OX, OX Open-Xchange, open xchange and OXtender
- *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the OX Software GmbH group of companies.
- *    The use of the Logos is not covered by the GNU General Public License.
- *    Instead, you are allowed to use these Logos according to the terms and
- *    conditions of the Creative Commons License, Version 2.5, Attribution,
- *    Non-commercial, ShareAlike, and the interpretation of the term
- *    Non-commercial applicable to the aforementioned license is published
- *    on the web site http://www.open-xchange.com/EN/legal/index.html.
- *
- *    Please make sure that third-party modules and libraries are used
- *    according to their respective licenses.
- *
- *    Any modifications to this package must retain all copyright notices
- *    of the original copyright holder(s) for the original code used.
- *
- *    After any such modifications, the original and derivative code shall remain
- *    under the copyright of the copyright holder(s) and/or original author(s)per
- *    the Attribution and Assignment Agreement that can be located at
- *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
- *    given Attribution for the derivative code and a license granting use.
- *
- *     Copyright (C) 2016-2020 OX Software GmbH
- *     Mail: info@open-xchange.com
- *
- *
- *     This program is free software; you can redistribute it and/or modify it
- *     under the terms of the GNU General Public License, Version 2 as published
- *     by the Free Software Foundation.
- *
- *     This program is distributed in the hope that it will be useful, but
- *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- *     for more details.
- *
- *     You should have received a copy of the GNU General Public License along
- *     with this program; if not, write to the Free Software Foundation, Inc., 59
- *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Any use of the work other than as authorized under this license or copyright law is prohibited.
  *
  */
 
 package com.openexchange.pns.loader.osgi;
 
 import static com.openexchange.osgi.Tools.withRanking;
+import com.openexchange.java.Strings;
 import java.io.IOException;
 import java.io.InputStream;
 import java.util.ArrayList;
@@ -67,8 +40,11 @@ import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import com.openexchange.pns.loader.ClientIdentifierProvider;
 import com.openexchange.pns.transport.apn.DefaultApnOptionsProvider;
+import com.openexchange.pns.transport.apns_http2.DefaultApnsHttp2OptionsProvider;
 import com.openexchange.pns.transport.apns_http2.util.ApnOptions;
 import com.openexchange.pns.transport.apns_http2.util.ApnOptionsProvider;
+import com.openexchange.pns.transport.apns_http2.util.ApnsHttp2Options;
+import com.openexchange.pns.transport.apns_http2.util.ApnsHttp2OptionsProvider;
 import com.openexchange.pns.transport.gcm.DefaultGcmOptionsProvider;
 import com.openexchange.pns.transport.gcm.GcmOptions;
 import com.openexchange.pns.transport.gcm.GcmOptionsProvider;
@@ -126,8 +102,8 @@ public class PushSecretsRegisterer implements ServiceTrackerCustomizer<ClientIde
 
         loadAndRegisterGcmOptions(client, props);
         loadAndRegisterApnOptions(client, props);
+        loadAndRegisterApnsHttp2Options(client, props);
 
-        // TODO APNS HTTP2
         // TODO Websockets
         // TODO wns
     }
@@ -185,6 +161,29 @@ public class PushSecretsRegisterer implements ServiceTrackerCustomizer<ClientIde
         }
     }
 
+    private void loadAndRegisterApnsHttp2Options(String client, Properties props) {
+        ServiceRegistration<?> registration = null;
+        try {
+            ApnsHttp2Options apnsHttp2Options = loadApnsHttp2OptionsFromFragment(client, props);
+            if (null == apnsHttp2Options) {
+                // Property keystore is missing and we assume that no APN certificate is wanted.
+                return;
+            }
+
+            DefaultApnsHttp2OptionsProvider apnsHttp2OptionsProvider = new DefaultApnsHttp2OptionsProvider(Collections.singletonMap(client, apnsHttp2Options));
+            registration = context.registerService(ApnsHttp2OptionsProvider.class, apnsHttp2OptionsProvider, withRanking(1000));
+            rememberRegistration(client, registration);
+            registration = null; // Everything went fine
+            LOG.info("Loaded APNs HTTP/2 push configuration for client identifier {}", client);
+        } catch (IOException e) {
+            LOG.error("Failed to load APNs HTTP/2 configuration for push client {}", client, e);
+        } finally {
+            if (null != registration) {
+                registration.unregister();
+            }
+        }
+    }
+
     private void rememberRegistration(String client, ServiceRegistration<?> registration) {
         List<ServiceRegistration<?>> list = serviceRegistrations.get(client);
         if (null == list) {
@@ -224,22 +223,80 @@ public class PushSecretsRegisterer implements ServiceTrackerCustomizer<ClientIde
     }
 
     private static ApnOptions loadApnOptionsFromFragment(String client, Properties props) throws IOException {
-        String filename = props.getProperty("apnKeystoreFile");
-        if (null == filename) {
-            return null;
-        }
-
-        String password = props.getProperty("apnPassword");
-        if (null == password) {
-            throw new IOException(client + ".properties is missing property password for keystore password.");
-        }
-
-        boolean production = Boolean.parseBoolean(props.getProperty("apnProduction", Boolean.TRUE.toString()));
-        try (InputStream is = PushSecretsRegisterer.class.getClassLoader().getResourceAsStream(filename);) {
-            if (null == is) {
-                throw new IOException("Could not load resource " + filename + " from com.openexchange.pns.loader fragment.");
+        String privateKeyFile = props.getProperty("apnPrivatekeyFile");
+        if (Strings.isNotEmpty(privateKeyFile)) {
+            String keyId = props.getProperty("apnKeyId");
+            String teamId = props.getProperty("apnTeamId");
+            if (null == keyId) {
+                throw new IOException(client + ".properties is missing property keyId.");
+            }
+            if (null == teamId) {
+                throw new IOException(client + ".properties is missing property teamId.");
+            }
+            boolean production = Boolean.parseBoolean(props.getProperty("apnProduction", Boolean.TRUE.toString()));
+            try (InputStream is = PushSecretsRegisterer.class.getClassLoader().getResourceAsStream(privateKeyFile);) {
+                if (null == is) {
+                    throw new IOException("Could not load resource " + privateKeyFile + " from com.openexchange.pns.loader fragment.");
+                }
+                return new ApnOptions(IOUtils.toByteArray(is), keyId, teamId, production, TOPIC_IOS_VANILLA_MAILAPP, client);
             }
-            return new ApnOptions(IOUtils.toByteArray(is), password, production, TOPIC_IOS_VANILLA_MAILAPP, client);
         }
+        
+        String filename = props.getProperty("apnKeystoreFile");
+        if (Strings.isNotEmpty(filename)) {
+
+            String password = props.getProperty("apnPassword");
+            if (null == password) {
+                throw new IOException(client + ".properties is missing property password for keystore password.");
+            }
+
+            boolean production = Boolean.parseBoolean(props.getProperty("apnProduction", Boolean.TRUE.toString()));
+            try (InputStream is = PushSecretsRegisterer.class.getClassLoader().getResourceAsStream(filename);) {
+                if (null == is) {
+                    throw new IOException("Could not load resource " + filename + " from com.openexchange.pns.loader fragment.");
+                }
+                return new ApnOptions(IOUtils.toByteArray(is), password, production, TOPIC_IOS_VANILLA_MAILAPP, client);
+            }
+        }
+        return null;
+    }
+
+    private static ApnsHttp2Options loadApnsHttp2OptionsFromFragment(String client, Properties props) throws IOException {
+        String privateKeyFile = props.getProperty("apnsHttp2PrivatekeyFile");
+        if (Strings.isNotEmpty(privateKeyFile)) {
+            String keyId = props.getProperty("apnsHttp2KeyId");
+            String teamId = props.getProperty("apnsHttp2TeamId");
+            if (null == keyId) {
+                throw new IOException(client + ".properties is missing property keyId.");
+            }
+            if (null == teamId) {
+                throw new IOException(client + ".properties is missing property teamId.");
+            }
+            boolean production = Boolean.parseBoolean(props.getProperty("apnsHttp2Production", Boolean.TRUE.toString()));
+            try (InputStream is = PushSecretsRegisterer.class.getClassLoader().getResourceAsStream(privateKeyFile);) {
+                if (null == is) {
+                    throw new IOException("Could not load resource " + privateKeyFile + " from com.openexchange.pns.loader fragment.");
+                }
+                return new ApnsHttp2Options(client, IOUtils.toByteArray(is), keyId, teamId, production, TOPIC_IOS_VANILLA_MAILAPP);
+            }
+        }
+        
+        String filename = props.getProperty("apnsHttp2KeystoreFile");
+        if (Strings.isNotEmpty(filename)) {
+
+            String password = props.getProperty("apnsHttp2Password");
+            if (null == password) {
+                throw new IOException(client + ".properties is missing property password for keystore password.");
+            }
+
+            boolean production = Boolean.parseBoolean(props.getProperty("apnsHttp2Production", Boolean.TRUE.toString()));
+            try (InputStream is = PushSecretsRegisterer.class.getClassLoader().getResourceAsStream(filename);) {
+                if (null == is) {
+                    throw new IOException("Could not load resource " + filename + " from com.openexchange.pns.loader fragment.");
+                }
+                return new ApnsHttp2Options(client, IOUtils.toByteArray(is), password, production, TOPIC_IOS_VANILLA_MAILAPP);
+            }
+        }
+        return null;
     }
 }
