@@ -1,49 +1,21 @@
 /*
+ * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @license AGPL-3.0
  *
- *    OPEN-XCHANGE legal information
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *    All intellectual property rights in the Software are protected by
- *    international copyright laws.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
  *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with OX App Suite.  If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
  *
- *    In some countries OX, OX Open-Xchange, open xchange and OXtender
- *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the OX Software GmbH. group of companies.
- *    The use of the Logos is not covered by the GNU General Public License.
- *    Instead, you are allowed to use these Logos according to the terms and
- *    conditions of the Creative Commons License, Version 2.5, Attribution,
- *    Non-commercial, ShareAlike, and the interpretation of the term
- *    Non-commercial applicable to the aforementioned license is published
- *    on the web site http://www.open-xchange.com/EN/legal/index.html.
- *
- *    Please make sure that third-party modules and libraries are used
- *    according to their respective licenses.
- *
- *    Any modifications to this package must retain all copyright notices
- *    of the original copyright holder(s) for the original code used.
- *
- *    After any such modifications, the original and derivative code shall remain
- *    under the copyright of the copyright holder(s) and/or original author(s)per
- *    the Attribution and Assignment Agreement that can be located at
- *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
- *    given Attribution for the derivative code and a license granting use.
- *
- *     Copyright (C) 2016-2020 OX Software GmbH
- *     Mail: info@open-xchange.com
- *
- *
- *     This program is free software; you can redistribute it and/or modify it
- *     under the terms of the GNU General Public License, Version 2 as published
- *     by the Free Software Foundation.
- *
- *     This program is distributed in the hope that it will be useful, but
- *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- *     for more details.
- *
- *     You should have received a copy of the GNU General Public License along
- *     with this program; if not, write to the Free Software Foundation, Inc., 59
- *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Any use of the work other than as authorized under this license or copyright law is prohibited.
  *
  */
 
@@ -52,6 +24,7 @@ package com.openexchange.pns.transport.apn.osgi;
 import static com.openexchange.osgi.Tools.withRanking;
 import java.io.File;
 import java.io.FileInputStream;
+import java.io.IOException;
 import java.io.InputStream;
 import java.util.LinkedHashMap;
 import java.util.Map;
@@ -64,7 +37,6 @@ import com.openexchange.config.ForcedReloadable;
 import com.openexchange.config.Interests;
 import com.openexchange.config.Reloadable;
 import com.openexchange.config.cascade.ConfigViewFactory;
-import com.openexchange.java.Streams;
 import com.openexchange.java.Strings;
 import com.openexchange.osgi.HousekeepingActivator;
 import com.openexchange.pns.PushExceptionCodes;
@@ -73,9 +45,9 @@ import com.openexchange.pns.PushSubscriptionRegistry;
 import com.openexchange.pns.transport.apn.DefaultApnOptionsProvider;
 import com.openexchange.pns.transport.apn.internal.ApnPushNotificationTransport;
 import com.openexchange.pns.transport.apns_http2.util.ApnOptions;
+import com.openexchange.pns.transport.apns_http2.util.ApnOptions.AuthType;
 import com.openexchange.pns.transport.apns_http2.util.ApnOptionsProvider;
 
-
 /**
  * {@link ApnPushNotificationTransportActivator}
  *
@@ -109,10 +81,7 @@ public class ApnPushNotificationTransportActivator extends HousekeepingActivator
 
     @Override
     public Interests getInterests() {
-        return DefaultInterests.builder()
-            .configFileNames(CONFIGFILE_APNS_OPTIONS)
-            .propertiesOfInterest("com.openexchange.pns.transport.apn.ios.enabled")
-            .build();
+        return DefaultInterests.builder().configFileNames(CONFIGFILE_APNS_OPTIONS).propertiesOfInterest("com.openexchange.pns.transport.apn.ios.enabled").build();
     }
 
     @Override
@@ -206,43 +175,108 @@ public class ApnPushNotificationTransportActivator extends HousekeepingActivator
 
             // Enabled?
             Boolean enabled = getBooleanOption("enabled", Boolean.TRUE, values);
-            if (enabled.booleanValue()) {
-                // Keystore name
-                String keystoreName = getStringOption("keystore", values);
-                if (Strings.isEmpty(keystoreName)) {
-                    LOG.info("Missing \"keystore\" APNS option for client {}. Ignoring that client's configuration.", client);
-                } else {
-                    String password = getStringOption("password", values);
-                    if (Strings.isEmpty(password)) {
-                        LOG.info("Missing \"password\" APNS option for client {}. Ignoring that client's configuration.", client);
-                    } else {
-                        String topic = getStringOption("topic", values);
-                        if (Strings.isEmpty(topic)) {
-                            LOG.info("Missing \"topic\" APNS option for client {}. Ignoring that client's configuration.", client);
-                        } else {
-                            Boolean production = getBooleanOption("production", Boolean.TRUE, values);
-                            ApnOptions apnOptions;
-                            try {
-                                apnOptions = createOptions(keystoreName, password, production.booleanValue(), topic, client);
-                            } catch (Exception e) {
-                                apnOptions = null;
-                                LOG.warn("Failed to parse APNS options for client {}.", client, e);
-                            }
-                            if (null != apnOptions) {
-                                options.put(client, apnOptions);
-                                LOG.info("Parsed APNS options for client {}.", client);
-                            }
-                        }
-                    }
-                }
-            } else {
+            if (false == enabled.booleanValue()) {
                 LOG.info("APNS options for client {} is disabled.", client);
+                continue;
+            }
+
+            // Auth type
+            AuthType authType = AuthType.authTypeFor(getStringOption("authtype", values));
+            if (null == authType) {
+                // Missing or invalid auth type. Assume "certificate" as fall-back.
+                LOG.debug("Missing or invalid authentication type in APNS HTTP/2 options for client {}. Using fall-back \"certificate\".", client);
+                authType = AuthType.CERTIFICATE;
+            }
+            switch (authType) {
+                case CERTIFICATE:
+                    processCertificate(client, options, values);
+                    break;
+                case JWT:
+                    processJWT(client, options, values);
+                    break;
+                default:
+                    LOG.debug("Unsupported auth type {}", authType);
             }
         }
         return options;
     }
 
-    private Boolean getBooleanOption(String name, Boolean def, Map<String, Object> values) {
+    /**
+     * Process the {@link AuthType#CERTIFICATE}
+     *
+     * @param client The client
+     * @param options the options
+     * @param values The values
+     * @return <code>true</code> if appropriate options have been initialized for given client; otherwise <code>false</code>
+     */
+    private static boolean processCertificate(String client, Map<String, ApnOptions> options, Map<String, Object> values) {
+        String keystoreName = getStringOption("keystore", values);
+        if (Strings.isEmpty(keystoreName)) {
+            LOG.info("Missing \"keystore\" APNS option for client {}. Ignoring that client's configuration.", client);
+            return false;
+        }
+        String password = getStringOption("password", values);
+        if (Strings.isEmpty(password)) {
+            LOG.info("Missing \"password\" APNS option for client {}. Ignoring that client's configuration.", client);
+            return false;
+        }
+        String topic = getStringOption("topic", values);
+        if (Strings.isEmpty(topic)) {
+            LOG.info("Missing \"topic\" APNS option for client {}. Ignoring that client's configuration.", client);
+            return false;
+        }
+        Boolean production = getBooleanOption("production", Boolean.TRUE, values);
+        try {
+            options.put(client, new ApnOptions(getResourceBytes(keystoreName), password, production.booleanValue(), topic, client));
+            LOG.info("Parsed APNS options for client {}.", client);
+            return true;
+        } catch (Exception e) {
+            LOG.warn("Failed to parse APNS options for client {}.", client, e);
+        }
+        return false;
+    }
+
+    /**
+     * Process the {@link AuthType#JWT}
+     *
+     * @param client The client
+     * @param options the options
+     * @param values The values
+     * @return <code>true</code> if appropriate options have been initialized for given client; otherwise <code>false</code>
+     */
+    private static boolean processJWT(String client, Map<String, ApnOptions> options, Map<String, Object> values) {
+        String privateKeyName = getStringOption("privatekey", values);
+        if (Strings.isEmpty(privateKeyName)) {
+            LOG.info("Missing \"privatekey\" APNS option for client {}. Ignoring that client's configuration.", client);
+            return false;
+        }
+        String keyId = getStringOption("keyid", values);
+        if (Strings.isEmpty(keyId)) {
+            LOG.info("Missing \"keyid\" APNS option for client {}. Ignoring that client's configuration.", client);
+            return false;
+        }
+        String teamId = getStringOption("teamid", values);
+        if (Strings.isEmpty(teamId)) {
+            LOG.info("Missing \"teamid\" APNS option for client {}. Ignoring that client's configuration.", client);
+            return false;
+        }
+        String topic = getStringOption("topic", values);
+        if (Strings.isEmpty(topic)) {
+            LOG.info("Missing \"topic\" APNS option for client {}. Ignoring that client's configuration.", client);
+            return false;
+        }
+        Boolean production = getBooleanOption("production", Boolean.TRUE, values);
+        try {
+            options.put(client, new ApnOptions(getResourceBytes(privateKeyName), keyId, teamId, production.booleanValue(), topic, client));
+            LOG.info("Parsed APNS options for client {}.", client);
+            return true;
+        } catch (Exception e) {
+            LOG.warn("Failed to parse APNS options for client {}.", client, e);
+        }
+        return false;
+    }
+
+    private static Boolean getBooleanOption(String name, Boolean def, Map<String, Object> values) {
         Object object = values.get(name);
         if (object instanceof Boolean) {
             return (Boolean) object;
@@ -250,7 +284,7 @@ public class ApnPushNotificationTransportActivator extends HousekeepingActivator
         return null == object ? def : Boolean.valueOf(object.toString());
     }
 
-    private String getStringOption(String name, Map<String, Object> values) {
+    private static String getStringOption(String name, Map<String, Object> values) {
         Object object = values.get(name);
         if (null == object) {
             return null;
@@ -259,14 +293,9 @@ public class ApnPushNotificationTransportActivator extends HousekeepingActivator
         return Strings.isEmpty(str) ? null : str.trim();
     }
 
-    private ApnOptions createOptions(String resourceName, String password, boolean production, String topic, String client) throws Exception {
-        InputStream resourceStream = null;
-        try {
-            resourceStream = new FileInputStream(new File(resourceName));
-            return new ApnOptions(IOUtils.toByteArray(resourceStream), password, production, topic, client);
-        } finally {
-            Streams.close(resourceStream);
+    private static byte[] getResourceBytes(String resourceName) throws IOException {
+        try (InputStream resourceStream = new FileInputStream(new File(resourceName))) {
+            return IOUtils.toByteArray(resourceStream);
         }
     }
-
 }
