@@ -1,49 +1,21 @@
 /*
+ * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @license AGPL-3.0
  *
- *    OPEN-XCHANGE legal information
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *    All intellectual property rights in the Software are protected by
- *    international copyright laws.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
  *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with OX App Suite.  If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
  *
- *    In some countries OX, OX Open-Xchange, open xchange and OXtender
- *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the OX Software GmbH. group of companies.
- *    The use of the Logos is not covered by the GNU General Public License.
- *    Instead, you are allowed to use these Logos according to the terms and
- *    conditions of the Creative Commons License, Version 2.5, Attribution,
- *    Non-commercial, ShareAlike, and the interpretation of the term
- *    Non-commercial applicable to the aforementioned license is published
- *    on the web site http://www.open-xchange.com/EN/legal/index.html.
- *
- *    Please make sure that third-party modules and libraries are used
- *    according to their respective licenses.
- *
- *    Any modifications to this package must retain all copyright notices
- *    of the original copyright holder(s) for the original code used.
- *
- *    After any such modifications, the original and derivative code shall remain
- *    under the copyright of the copyright holder(s) and/or original author(s)per
- *    the Attribution and Assignment Agreement that can be located at
- *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
- *    given Attribution for the derivative code and a license granting use.
- *
- *     Copyright (C) 2016-2020 OX Software GmbH
- *     Mail: info@open-xchange.com
- *
- *
- *     This program is free software; you can redistribute it and/or modify it
- *     under the terms of the GNU General Public License, Version 2 as published
- *     by the Free Software Foundation.
- *
- *     This program is distributed in the hope that it will be useful, but
- *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- *     for more details.
- *
- *     You should have received a copy of the GNU General Public License along
- *     with this program; if not, write to the Free Software Foundation, Inc., 59
- *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Any use of the work other than as authorized under this license or copyright law is prohibited.
  *
  */
 
@@ -53,11 +25,12 @@ import static com.openexchange.java.Autoboxing.I;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.HashMap;
+import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Map.Entry;
+import java.util.Set;
 import java.util.concurrent.ExecutionException;
-import java.util.stream.Collectors;
 import org.json.JSONArray;
 import org.json.JSONException;
 import org.json.JSONObject;
@@ -278,46 +251,68 @@ public class ApnsHttp2PushPerformer {
         }
 
         // Create mapping for device token to push match
-        Map<String, PushMatch> deviceToken2PushMatch = payloads.stream().collect(Collectors.toMap(entry -> entry.getValue().getToken(), entry -> entry.getKey()));
+        Map<String, Set<PushMatch>> deviceToken2PushMatch = new HashMap<>(payloads.size());
+        for (Map.Entry<PushMatch, ApnsPushNotification> payload : payloads) {
+            String token = payload.getValue().getToken();
+            Set<PushMatch> matches = deviceToken2PushMatch.get(token);
+            if (matches == null) {
+                matches = new LinkedHashSet<>(2);
+                deviceToken2PushMatch.put(token, matches);
+            }
+            matches.add(payload.getKey());
+        }
 
         // Process results
-        for (NotificationResponsePerDevice notificationPerDevice : notifications) {
+        NextNotification: for (NotificationResponsePerDevice notificationPerDevice : notifications) {
             PushNotificationFuture<ApnsPushNotification, PushNotificationResponse<ApnsPushNotification>> sendNotificationFuture = notificationPerDevice.sendNotificationFuture;
             String deviceToken = notificationPerDevice.deviceToken;
-            PushMatch pushMatch = deviceToken == null ? null : deviceToken2PushMatch.get(deviceToken);
-            try {
-                PushNotificationResponse<ApnsPushNotification> pushNotificationResponse = sendNotificationFuture.get();
-                if (pushNotificationResponse.isAccepted()) {
-                    LOG.debug("Push notification accepted by APNs gateway for device token: {}", deviceToken);
-                    if (pushMatch != null) {
-                        LOG.info("Sent notification \"{}\" via transport '{}' for user {} in context {} to device token: {}", pushMatch.getTopic(), ID, I(pushMatch.getUserId()), I(pushMatch.getContextId()), deviceToken);
-                    }
-                } else {
-                    if (pushNotificationResponse.getTokenInvalidationTimestamp() != null || isInvalidToken(pushNotificationResponse.getRejectionReason())) {
-                        LOG.warn("Unsuccessful push notification due to inactive or invalid device token: {}", deviceToken);
-                        if (null != pushMatch) {
-                            boolean removed = removeSubscription(pushMatch);
-                            if (removed) {
-                                LOG.info("Removed subscription for device with token: {}.", pushMatch.getToken());
-                            } else {
-                                LOG.debug("Could not remove subscriptions for device with token: {}.", pushMatch.getToken());
-                            }
-                        } else {
-                            int removed = removeSubscriptions(deviceToken);
-                            if (0 < removed) {
-                                LOG.info("Removed {} subscriptions for device with token: {}.", Integer.valueOf(removed), deviceToken);
-                            }
+            if (deviceToken == null) {
+                // Missing device token
+                continue NextNotification;
+            }
+
+            Set<PushMatch> pushMatches = deviceToken2PushMatch.get(deviceToken);
+            if (pushMatches == null) {
+                // No push matches available for current device token
+                continue NextNotification;
+            }
+
+            // Process push matches
+            for (PushMatch pushMatch : pushMatches) {
+                try {
+                    PushNotificationResponse<ApnsPushNotification> pushNotificationResponse = sendNotificationFuture.get();
+                    if (pushNotificationResponse.isAccepted()) {
+                        LOG.debug("Push notification accepted by APNs gateway for device token: {}", deviceToken);
+                        if (pushMatch != null) {
+                            LOG.info("Sent notification \"{}\" via transport '{}' for user {} in context {} to device token: {}", pushMatch.getTopic(), ID, I(pushMatch.getUserId()), I(pushMatch.getContextId()), deviceToken);
                         }
                     } else {
-                        LOG.warn("Unsuccessful push notification for device with token: {}", deviceToken);
+                        if (pushNotificationResponse.getTokenInvalidationTimestamp() != null || isInvalidToken(pushNotificationResponse.getRejectionReason())) {
+                            LOG.warn("Unsuccessful push notification due to inactive or invalid device token: {}", deviceToken);
+                            if (null != pushMatch) {
+                                boolean removed = removeSubscription(pushMatch);
+                                if (removed) {
+                                    LOG.info("Removed subscription for device with token: {}.", pushMatch.getToken());
+                                } else {
+                                    LOG.debug("Could not remove subscriptions for device with token: {}.", pushMatch.getToken());
+                                }
+                            } else {
+                                int removed = removeSubscriptions(deviceToken);
+                                if (0 < removed) {
+                                    LOG.info("Removed {} subscriptions for device with token: {}.", Integer.valueOf(removed), deviceToken);
+                                }
+                            }
+                        } else {
+                            LOG.warn("Unsuccessful push notification for device with token: {}", deviceToken);
+                        }
                     }
+                } catch (ExecutionException e) {
+                    LOG.warn("Failed to send push notification for device token {}", deviceToken, e.getCause());
+                } catch (InterruptedException e) {
+                    Thread.currentThread().interrupt();
+                    LOG.warn("Interrupted while sending push notification for device token {}", deviceToken, e.getCause());
+                    return;
                 }
-            } catch (ExecutionException e) {
-                LOG.warn("Failed to send push notification for device token {}", deviceToken, e.getCause());
-            } catch (InterruptedException e) {
-                Thread.currentThread().interrupt();
-                LOG.warn("Interrupted while sending push notification for device token {}", deviceToken, e.getCause());
-                return;
             }
         }
     }
